﻿/* Alpha Mix Effect
 * - Set alpha to different channels, or intensity then add or scale value.
 * 
 * Programmer: Matthew Reynolds
 * 
 * v 1.0 - April 10 2009
 * 
 */


using System;
using System.Text;
using System.Reflection;
using PaintDotNet;
using PaintDotNet.Effects;
using PaintDotNet.IndirectUI;
using PaintDotNet.PropertySystem;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Text;
using System.Runtime.InteropServices;


namespace Matthew
{
    [PluginSupportInfo(typeof(PluginSupportInfo), DisplayName = "MattAlphaEffect")]
    public class AlphaMixEffect : PropertyBasedEffect
    {
        public enum PropertyNames
        {
            AlphaSource, //default, r,g,b,or intensity
            SetAlpha,
            SetAlphaAmount,
            ScaleAlpha,
            ScaleAlphaAmount,
            AddAlpha,
            AddAlphaAmount,
            Invert,
        }

        //static
        static string SETALPHA_DEFAULT = "Use Default";
        static string SETALPHA_SET = "Set Value";
        static string SETALPHA_R = "Set From R Channel";
        static string SETALPHA_G = "Set From G Channel";
        static string SETALPHA_B = "Set From B Channel";
        static string SETALPHA_I = "Set From Intensity";

        //properties
        bool setFlag = false;
        bool setFromRChannel = false;
        bool setFromGChannel = false;
        bool setFromBChannel = false;
        bool setFromIntensity = false;
        bool invert = false;
        bool addFlag = false;
        bool scaleFlag = false;
        int amountSet = 0;
        int amountAdd = 0;
        double amountScale = 1.0f;

        public static string StaticName
        {
            get
            {
                return "Alpha Mixer";
            }
        }

        public static Image StaticIcon
        {
            get
            {
                Bitmap bm = null;
                Assembly a = Assembly.GetExecutingAssembly();

                using (System.IO.Stream bmStream = a.GetManifestResourceStream("Matthew.Icons.AlphaMixIcon.png"))
                {
                    bm = new Bitmap(bmStream);
                }
                return bm;
            }
        }

        public AlphaMixEffect()
            : base(StaticName, StaticIcon, SubmenuNames.Photo, EffectFlags.Configurable)
        {
        }

        protected override PropertyCollection OnCreatePropertyCollection()
        {
            List<Property> props = new List<Property>();
      
            props.Add(new StaticListChoiceProperty(
                PropertyNames.AlphaSource,
                new string[] { 
                    SETALPHA_DEFAULT,
                    SETALPHA_SET,
                    SETALPHA_R, 
                    SETALPHA_G,
                    SETALPHA_B,
                    SETALPHA_I},
                    0));

            //props.Add(new BooleanProperty(PropertyNames.SetAlpha, false));
            props.Add(new Int32Property(PropertyNames.SetAlphaAmount, 0, 0, 255));
            props.Add(new BooleanProperty(PropertyNames.AddAlpha, false));
            props.Add(new Int32Property(PropertyNames.AddAlphaAmount, 0, -255, 255));          
            props.Add(new BooleanProperty(PropertyNames.ScaleAlpha, false));
            props.Add(new DoubleProperty(PropertyNames.ScaleAlphaAmount, 1.0, 0, 20));
            props.Add(new BooleanProperty(PropertyNames.Invert, false));
            return new PropertyCollection(props);
        }

        protected override ControlInfo OnCreateConfigUI(PropertyCollection props)
        {
            ControlInfo configUI = CreateDefaultConfigUI(props);

            configUI.SetPropertyControlValue(PropertyNames.SetAlphaAmount, ControlInfoPropertyNames.SliderLargeChange, 5);
            configUI.SetPropertyControlValue(PropertyNames.AddAlphaAmount, ControlInfoPropertyNames.SliderSmallChange, 5);
            configUI.SetPropertyControlValue(PropertyNames.ScaleAlphaAmount, ControlInfoPropertyNames.SliderLargeChange, 1.0);
            configUI.SetPropertyControlValue(PropertyNames.ScaleAlphaAmount, ControlInfoPropertyNames.SliderSmallChange, 0.1);

            configUI.SetPropertyControlValue(PropertyNames.AlphaSource, ControlInfoPropertyNames.DisplayName, "ALPHA SOURCE");
            configUI.SetPropertyControlValue(PropertyNames.AlphaSource, ControlInfoPropertyNames.Description, ""); //"" keep divider from showing up

            configUI.SetPropertyControlValue(PropertyNames.SetAlphaAmount, ControlInfoPropertyNames.DisplayName, "ALPHA BASE");
            configUI.SetPropertyControlValue(PropertyNames.SetAlphaAmount, ControlInfoPropertyNames.Description, "Amount to set alpha (a = x)");

            configUI.SetPropertyControlValue(PropertyNames.AddAlpha, ControlInfoPropertyNames.DisplayName, "ALPHA ADDITION");
            configUI.SetPropertyControlValue(PropertyNames.AddAlpha, ControlInfoPropertyNames.Description, "Enabled");
            configUI.SetPropertyControlValue(PropertyNames.AddAlphaAmount, ControlInfoPropertyNames.DisplayName, "");
            configUI.SetPropertyControlValue(PropertyNames.AddAlphaAmount, ControlInfoPropertyNames.Description, "Amount to add to alpha (a = a + x)");

            configUI.SetPropertyControlValue(PropertyNames.ScaleAlpha, ControlInfoPropertyNames.DisplayName, "ALPHA SCALING");
            configUI.SetPropertyControlValue(PropertyNames.ScaleAlpha, ControlInfoPropertyNames.Description, "Enabled");
            configUI.SetPropertyControlValue(PropertyNames.ScaleAlphaAmount, ControlInfoPropertyNames.DisplayName, "");
            configUI.SetPropertyControlValue(PropertyNames.ScaleAlphaAmount, ControlInfoPropertyNames.Description, "Amount to scale alpha (a = x*a)");

            configUI.SetPropertyControlValue(PropertyNames.Invert, ControlInfoPropertyNames.DisplayName, "OTHER");
            configUI.SetPropertyControlValue(PropertyNames.Invert, ControlInfoPropertyNames.Description, "Invert Alpha (a = 1 - a)");
            configUI.SetPropertyControlValue(PropertyNames.Invert, ControlInfoPropertyNames.DisplayName, "Enable");

            return configUI;
        }

        protected override void OnSetRenderInfo(PropertyBasedEffectConfigToken newToken, RenderArgs dstArgs, RenderArgs srcArgs)
        {
            string choice = (string)newToken.GetProperty<StaticListChoiceProperty>(PropertyNames.AlphaSource).Value;
            setFlag = (choice == SETALPHA_SET);
            setFromRChannel = (choice == SETALPHA_R);
            setFromGChannel = (choice == SETALPHA_G);
            setFromBChannel = (choice == SETALPHA_B);
            setFromIntensity =  (choice == SETALPHA_I);
            
            this.invert = newToken.GetProperty<BooleanProperty>(PropertyNames.Invert).Value;
            this.addFlag = newToken.GetProperty<BooleanProperty>(PropertyNames.AddAlpha).Value;
            this.scaleFlag = newToken.GetProperty<BooleanProperty>(PropertyNames.ScaleAlpha).Value;
            this.amountSet = newToken.GetProperty<Int32Property>(PropertyNames.SetAlphaAmount).Value;
            this.amountAdd = newToken.GetProperty<Int32Property>(PropertyNames.AddAlphaAmount).Value;
            this.amountScale = newToken.GetProperty<DoubleProperty>(PropertyNames.ScaleAlphaAmount).Value;

            base.OnSetRenderInfo(newToken, dstArgs, srcArgs);
        }

        protected override void OnRender(Rectangle[] rois, int startIndex, int length)
        {
            if (length == 0) return;
            for (int i = startIndex; i < startIndex + length; ++i)
            {
                Render(DstArgs.Surface, SrcArgs.Surface, rois[i]);
            }
        }

        protected override void OnCustomizeConfigUIWindowProperties(PropertyCollection props)
        {
            // Change the effect's window title
            props[ControlInfoPropertyNames.WindowTitle].Value = "Matts Alpha Adjust v1.0";
            base.OnCustomizeConfigUIWindowProperties(props);
        }

        

        void Render(Surface dst, Surface src, Rectangle rc)
        {
            for (int y = rc.Top; y < rc.Bottom; ++y)
            {
                for (int x = rc.Left; x < rc.Right; ++x)
                {
                    
                    ColorBgra color = src[x, y];

                    //maybe optimize better, logic below
                    double alpha = setFlag ? amountSet : 
                        setFromIntensity ? color.GetIntensityByte() :
                        setFromRChannel ? color.R :
                        setFromGChannel ? color.G : 
                        setFromBChannel ? color.B : 
                        color.A;

                    /*
                    if (setFlag) alpha = amountSet;
                    else if(setFromIntensity) alpha = color.GetIntensityByte();
                    else if (setFromBChannel) alpha = color.B;
                    else if (setFromGChannel) alpha = color.G;
                    else if (setFromRChannel) alpha = color.R;
                    els  alpha = color.A;
                     */

                    if (addFlag) alpha += amountAdd;
                    if (scaleFlag) alpha *= amountScale;
                    if (!invert)
                    { //CLAMP 0 - 255
                        color.A = (byte)(alpha < 0 ? 0 :
                            alpha > 255 ? 255 :
                            alpha);
                    }
                    else
                    {
                        color.A = (byte)(alpha < 0 ? 255 :
                            alpha > 255 ? 0 :
                            255 - alpha);
                    }

                    dst[x,y] = color;
                }
            }
        }
    }
}