﻿/* Alpha Mix Effect
 * - Mix and blend color channels.  Set value from channels, or intensity then add, scale, and invert value.
 * - Change order channels are processed.
 * 
 * Note:
 * I ended up using delegates (Funtion Pointers) to speed things up. Using delegates I could speed rendering
 * up by reducing the amount of switch and if statments in the for loops.
 * 
 * Programmer: Matthew Reynolds
 * 
 * v 1.0 - April 10 2009
 * 
 */

using System;
using System.Drawing;
using System.Collections;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;
using PaintDotNet;
using PaintDotNet.Effects;

namespace Matthew.ChannelMixEffect
{
    public class ChannelMixEffect : PaintDotNet.Effects.Effect<ChannelMixToken>, PaintDotNet.IPluginSupportInfo
    {
        public static Image StaticIcon
        {
            get
            {
                Bitmap bm = null;
                Assembly a = Assembly.GetExecutingAssembly();

                using (System.IO.Stream bmStream = a.GetManifestResourceStream("Matthew.Icons.ChannelMixIcon.png"))

                {
                    bm = new Bitmap(bmStream);
                }
                return bm;
            }
        }

        public ChannelMixEffect()
            : base("Channel Mix Effect", StaticIcon, SubmenuNames.Photo, EffectFlags.Configurable)
        {
        }

        public override EffectConfigDialog CreateConfigDialog()
        {
            return new ChannelMixDialog();
        }
        protected override void OnSetRenderInfo(ChannelMixToken newToken, RenderArgs dstArgs, RenderArgs srcArgs)
        {
            base.OnSetRenderInfo(newToken, dstArgs, srcArgs);
        }

        protected override void OnRender(Rectangle[] renderRects, int startIndex, int length)
        {
            if (length == 0) return;
            for (int i = startIndex; i < startIndex + length; ++i)
            {
                Render(DstArgs.Surface, SrcArgs.Surface, renderRects[i]);
            }
        }


        //GET FUNCTIONS
        unsafe delegate double GetChannelVal(ColorBgra* c, ChannelOptions o);
        unsafe double funcGetA(ColorBgra* c, ChannelOptions o) { return c->A; }
        unsafe double funcGetR(ColorBgra* c, ChannelOptions o) { return c->R; }
        unsafe double funcGetG(ColorBgra* c, ChannelOptions o) { return c->G; }
        unsafe double funcGetB(ColorBgra* c, ChannelOptions o) { return c->B; }
        unsafe double funcGetIntensity(ColorBgra* c, ChannelOptions o) { return c->GetIntensityByte(); }
        unsafe double funcGetSet(ColorBgra* c, ChannelOptions o) { return o.set; }

        //returns pointer to function that gets value from channel
        unsafe GetChannelVal GetSourceFunc(ChannelOptions copt)
        {
            switch (copt.source)
            {
                case SourceEnum.Set: return funcGetSet;
                case SourceEnum.Intensity: return funcGetIntensity;
                case SourceEnum.Alpha: return funcGetA;
                case SourceEnum.Red: return funcGetR;
                case SourceEnum.Green: return funcGetG;
                case SourceEnum.Blue: return funcGetB;

                case SourceEnum.Default:
                    switch (copt.channel)
                    {
                        case ChannelsEnum.Alpha: return funcGetA;
                        case ChannelsEnum.Red: return funcGetR;
                        case ChannelsEnum.Green: return funcGetG;
                        case ChannelsEnum.Blue: return funcGetB;
                    }
                    break;
            }

            return funcGetSet;
        }

        //SET FUNCTIONS
        unsafe delegate void SetChannelVal(ColorBgra* c, double v);
        unsafe void funcSetA(ColorBgra* c, double v) { c->A = (byte)v; }
        unsafe void funcSetR(ColorBgra* c, double v) { c->R = (byte)v; }
        unsafe void funcSetG(ColorBgra* c, double v) { c->G = (byte)v; }
        unsafe void funcSetB(ColorBgra* c, double v) { c->B = (byte)v; }

        unsafe SetChannelVal GetDestFunc(ChannelOptions copt)
        {
            switch (copt.channel)
            {
                default:
                case ChannelsEnum.Alpha: return funcSetA;
                case ChannelsEnum.Red: return funcSetR;;
                case ChannelsEnum.Green: return funcSetG;
                case ChannelsEnum.Blue: return funcSetB;
            }
        }


        unsafe void Render(Surface dst, Surface src, Rectangle rc)
        {
            double value;

            //GetChannelVal gav;
            GetChannelVal gs1, gs2, gs3, gs4; //function pointers that get source value based on option
            SetChannelVal sd1, sd2, sd3, sd4; //function points to set value to channel
            SetChannelVal[] setFunc = new SetChannelVal[4];
            ChannelOptions[] opts = Token.processOrder;

            //function pointer used to avoid big switch statements in for loops, 
            gs1 = GetSourceFunc(opts[0]);
            gs2 = GetSourceFunc(opts[1]);
            gs3 = GetSourceFunc(opts[2]);
            gs4 = GetSourceFunc(opts[3]);
            sd1 = GetDestFunc(opts[0]);
            sd2 = GetDestFunc(opts[1]);
            sd3 = GetDestFunc(opts[2]);
            sd4 = GetDestFunc(opts[3]);


            for (int y = rc.Top; y < rc.Bottom; ++y)
            {
                for (int x = rc.Left; x < rc.Right; ++x)
                {
                    ColorBgra clrS = src[x, y];
                    ChannelOptions copt;

                    copt = opts[0];
                    if (copt.enable)
                    {
                        value = gs1(&clrS, copt);
                        if (copt.addEn) value += copt.add;
                        if (copt.scaleEn) value *= copt.scale;
                        if (copt.invert == false)
                        {
                            sd1(&clrS, value > 255 ? 255 : value < 0 ? 0 : value);
                        }
                        else
                        {
                            sd1(&clrS, value > 255 ? 0 : value < 0 ? 255 : 255 - value);
                        }
                    }

                    copt = opts[1];
                    if (copt.enable)
                    {
                        value = gs2(&clrS, copt);
                        if (copt.addEn) value += copt.add;
                        if (copt.scaleEn) value *= copt.scale;
                        if (copt.invert == false)
                        {
                            sd2(&clrS, value > 255 ? 255 : value < 0 ? 0 : value);
                        }
                        else
                        {
                            sd2(&clrS, value > 255 ? 0 : value < 0 ? 255 : 255 - value);
                        }
                    }

                    copt = opts[2];
                    if (copt.enable)
                    {
                        value = gs3(&clrS, copt);
                        if (copt.addEn) value += copt.add;
                        if (copt.scaleEn) value *= copt.scale;
                        if (copt.invert == false)
                        {
                            sd3(&clrS, value > 255 ? 255 : value < 0 ? 0 : value);
                        }
                        else
                        {
                            sd3(&clrS, value > 255 ? 0 : value < 0 ? 255 : 255 - value);
                        }
                    }

                    copt = opts[3];
                    if (copt.enable)
                    {
                        value = gs4(&clrS, copt);
                        if (copt.addEn) value += copt.add;
                        if (copt.scaleEn) value *= copt.scale;
                        if (copt.invert == false)
                        {
                            sd4(&clrS, value > 255 ? 255 : value < 0 ? 0 : value);
                        }
                        else
                        {
                            sd4(&clrS, value > 255 ? 0 : value < 0 ? 255 : 255 - value);
                        }
                    }

                    dst[x, y] = clrS;
                }
            }
        }
    

        #region IPluginSupportInfo Members
        Matthew.PluginSupportInfo support = new Matthew.PluginSupportInfo();
        string IPluginSupportInfo.Author
        {
            get { return support.Author; }
        }
        string IPluginSupportInfo.Copyright
        {
            get { return support.Copyright; }
        }

        string IPluginSupportInfo.DisplayName
        {
            get { return "Channel Mix Effect"; }
        }

        Version IPluginSupportInfo.Version
        {
            get { return new Version(1, 0); }
        }

        Uri IPluginSupportInfo.WebsiteUri
        {
            get { return support.WebsiteUri; }
        }

        #endregion
    }
}
